==> metlo-0.1.6.tgz: package/src/middlewares/fastify.ts <==
import { sendMetloTrace, shouldBlock } from "../rust_binds"
import { METLO_META } from "."
import { MetloConfig } from "../types"
import { Logger } from "../utils/logger"

export interface fastifyRequest {
  socket: any
  url: string
  query: any
  headers: object
  body: any
  method: string
}
export interface fastifyResponse {
  code(arg0: number): unknown
  send(): unknown
  getHeaders(): { [s: string]: unknown } | ArrayLike<unknown>
  statusCode: any
}

export interface MetloConfigFastify<
  _req extends fastifyRequest = fastifyRequest,
  _res extends fastifyResponse = fastifyResponse,
> extends MetloConfig {
  getUser?: (req: _req, res: _res) => string
  rejectResponse?: (req: _req, res: _res) => void
}

function versionCheck() {
  return true
}

const initialize = <
  _reqType extends fastifyRequest = fastifyRequest,
  _resType extends fastifyResponse = fastifyResponse,
>() => {
  const fp = require("fastify-plugin")
  if (!versionCheck()) {
    return
  }

  function compileRequest(request: _reqType, response: _resType) {
    return {
      url: {
        host: (
          METLO_META.opts?.apiHost || request.socket.remoteAddress || ""
        ).toString(),
        path: request.url.split("?")[0],
        parameters: Object.entries(request.query).map(([k, v]) => ({
          name: k.toString(),
          value: v.toString(),
        })),
      },
      headers: Object.entries(request.headers).map(([k, v]) => ({
        name: k.toString(),
        value: v.toString(),
      })),
      body: (typeof request.body === "object"
        ? JSON.stringify(request.body)
        : request.body || "{}"
      )
        .toString()
        .slice(0, METLO_META.maxBodyCaptureSize),
      method: request.method.toString(),
      user: METLO_META.getUser
        ? (METLO_META as MetloConfigFastify).getUser(request, response)
        : null,
    }
  }

  function compileMeta(request: _reqType, response: _resType) {
    return {
      environment: process.env.NODE_ENV || "development",
      incoming: true,
      source: request.socket.remoteAddress.toString(),
      sourcePort: request.socket.remotePort,
      destination: request.socket.localAddress.toString(),
      destinationPort: request.socket.localPort,
      metloSource: "node/fastify",
    }
  }

  function compileResponse(
    request: _reqType,
    response: _resType,
    responseBody,
  ) {
    return {
      status: response.statusCode,
      headers: Object.entries(response.getHeaders()).map(([k, v]) => ({
        name: k.toString(),
        value: v.toString(),
      })),
      body: (typeof responseBody === "object"
        ? JSON.stringify(responseBody)
        : responseBody || "{}"
      )
        .toString()
        .slice(0, METLO_META.maxBodyCaptureSize),
    }
  }

  async function compileInformation(
    request: _reqType,
    response: _resType,
    responseBody,
  ) {
    try {
      const data = JSON.stringify({
        request: compileRequest(request, response),
        response: compileResponse(request, response, responseBody),
        meta: compileMeta(request, response),
      })

      sendMetloTrace(data)
    } catch (e) {
      Logger().warn(`Metlo encountered an error ${e}`);
    }
  }

  if (METLO_META.metloEnabled) {
    const callback: (fastify: any, options: any, done: () => void) => void = fp(
      (fastify, options, done) => {
        try {
          fastify.addHook(
            "preHandler",
            (req: _reqType, res: _resType, next) => {
              try {
                if (!METLO_META.disableBlocking) {
                  try {
                    const apiReq = JSON.stringify(compileRequest(req, res))
                    const apiMeta = JSON.stringify(compileMeta(req, res))
                    const willBlock = shouldBlock(apiReq, apiMeta)
                    if (willBlock) {
                      if (METLO_META.rejectResponse) {
                        return (METLO_META as MetloConfigFastify).rejectResponse(
                          req,
                          res,
                        )
                      } else {
                        res.code(403)
                        return res.send()
                      }
                    }
                  } catch (e) {
                    Logger().warn(`Metlo encountered an error ${e}`);
                  }
                }
              } catch (err) {
                Logger().log("debug", err)
              }
              return next()
            },
          )

          fastify.addHook(
            "onSend",
            (req: _reqType, res: _resType, response, next) => {
              try {
                compileInformation(req, res, response)
              } catch (err) {
                Logger().log("debug", err)
              }
              return next()
            },
          )
        } catch (err) {
          Logger().log("debug", err)
        }
        return done()
      },
      { name: "metlo", encapsulate: false },
    )
    return callback
  } else {
    return dummyModule()
  }
}

export const dummyModule = () => {
  const fp = require("fastify-plugin")
  return fp((fastify, options, done) => {
    done()
  })
}

export default initialize

==> metlo-0.1.11.tgz: package/src/middlewares/fastify.ts <==
import { sendMetloTrace, shouldBlock } from "../rust_binds"
import { METLO_META } from "."
import { MetloConfig } from "../types"
import { Logger } from "../utils/logger"

export interface fastifyRequest {
  socket: any
  url: string
  query: any
  headers: object
  body: any
  method: string
}
export interface fastifyResponse {
  code(arg0: number): unknown
  send(): unknown
  getHeaders(): { [s: string]: unknown } | ArrayLike<unknown>
  statusCode: any
}

export interface MetloConfigFastify<
  _req extends fastifyRequest = fastifyRequest,
  _res extends fastifyResponse = fastifyResponse,
> extends MetloConfig {
  getUser?: (req: _req, res: _res) => string
  rejectResponse?: (req: _req, res: _res) => void
}

function versionCheck() {
  return true
}

const initialize = <
  _reqType extends fastifyRequest = fastifyRequest,
  _resType extends fastifyResponse = fastifyResponse,
>() => {
  const fp = require("fastify-plugin")
  if (!versionCheck()) {
    return
  }

  function compileRequest(request: _reqType, response: _resType) {
    return {
      url: {
        host: (
          METLO_META.opts?.apiHost || request.socket.remoteAddress || ""
        ).toString(),
        path: request.url.split("?")[0],
        parameters: Object.entries(request.query).map(([k, v]) => ({
          name: k.toString(),
          value: v.toString(),
        })),
      },
      headers: Object.entries(request.headers).map(([k, v]) => ({
        name: k.toString(),
        value: v.toString(),
      })),
      body: (typeof request.body === "object"
        ? JSON.stringify(request.body)
        : request.body || "{}"
      )
        .toString()
        .slice(0, METLO_META.maxBodyCaptureSize),
      method: request.method.toString(),
      user: METLO_META.getUser
        ? (METLO_META as MetloConfigFastify).getUser(request, response)
        : null,
    }
  }

  function compileMeta(request: _reqType, response: _resType) {
    return {
      environment: process.env.NODE_ENV || "development",
      incoming: true,
      source: (request.socket.remoteAddress || "").toString(),
      sourcePort: (request.socket.remotePort || 0),
      destination: (request.socket.localAddress || "").toString(),
      destinationPort: (request.socket.localPort || 0),
      metloSource: "node/fastify",
    }
  }

  function compileResponse(
    request: _reqType,
    response: _resType,
    responseBody,
  ) {
    return {
      status: response.statusCode,
      headers: Object.entries(response.getHeaders()).map(([k, v]) => ({
        name: k.toString(),
        value: v.toString(),
      })),
      body: (typeof responseBody === "object"
        ? JSON.stringify(responseBody)
        : responseBody || "{}"
      )
        .toString()
        .slice(0, METLO_META.maxBodyCaptureSize),
    }
  }

  async function compileInformation(
    request: _reqType,
    response: _resType,
    responseBody,
  ) {
    try {
      const data = JSON.stringify({
        request: compileRequest(request, response),
        response: compileResponse(request, response, responseBody),
        meta: compileMeta(request, response),
      })

      sendMetloTrace(data)
    } catch (e) {
      Logger().warn(`Metlo encountered an error ${e}`);
    }
  }

  if (METLO_META.metloEnabled) {
    const callback: (fastify: any, options: any, done: () => void) => void = fp(
      (fastify, options, done) => {
        try {
          fastify.addHook(
            "preHandler",
            (req: _reqType, res: _resType, next) => {
              try {
                if (!METLO_META.disableBlocking) {
                  try {
                    const apiReq = JSON.stringify(compileRequest(req, res))
                    const apiMeta = JSON.stringify(compileMeta(req, res))
                    const willBlock = shouldBlock(apiReq, apiMeta)
                    if (willBlock) {
                      if (METLO_META.rejectResponse) {
                        return (METLO_META as MetloConfigFastify).rejectResponse(
                          req,
                          res,
                        )
                      } else {
                        res.code(403)
                        return res.send()
                      }
                    }
                  } catch (e) {
                    Logger().warn(`Metlo encountered an error ${e}`);
                  }
                }
              } catch (err) {
                Logger().log("debug", err)
              }
              return next()
            },
          )

          fastify.addHook(
            "onSend",
            (req: _reqType, res: _resType, response, next) => {
              try {
                compileInformation(req, res, response)
              } catch (err) {
                Logger().log("debug", err)
              }
              return next()
            },
          )
        } catch (err) {
          Logger().log("debug", err)
        }
        return done()
      },
      { name: "metlo", encapsulate: false },
    )
    return callback
  } else {
    return dummyModule()
  }
}

export const dummyModule = () => {
  const fp = require("fastify-plugin")
  return fp((fastify, options, done) => {
    done()
  })
}

export default initialize
